<?php

if (!defined('ABSPATH')) exit;

function sap_script_playlist_templates() { ?>
<script type="text/html" id="tmpl-sap-playlist-current-item">
    <# if ( data.image ) { #>
    <img src="{{ data.thumb.src }}"/>
    <# } #>
    <div class="sap-playlist-caption">
        <span class="sap-playlist-item-meta sap-playlist-item-title">&#8220;{{ data.title }}&#8221;</span>
        <# if ( data.meta.artist ) { #><span class="sap-playlist-item-meta sap-playlist-item-artist">{{ data.meta.artist }}</span><# } #>
        <# if ( data.meta.album ) { #><span class="sap-playlist-item-meta sap-playlist-item-album">{{ data.meta.album }}</span><# } #>
        <# if ( data.meta.year ) { #><span class="sap-playlist-item-meta sap-playlist-item-year">{{ data.meta.year }}</span><# } #>
    </div>
</script>
<script type="text/html" id="tmpl-sap-playlist-item">
    <div class="sap-playlist-item">
        <a class="sap-playlist-item-play" href="{{ data.src }}">
            <# if ( data.tracks ) { #>
                <span class="sap-playlist-item-index">
                    {{ data.index ? ( data.index + '. ' ) : '' }}
                </span>
            <# } #>

            <# if ( data.caption ) { #>
                {{ data.caption }}
            <# } else { #>
                <span class="sap-playlist-item-title">&#8220;{{ data.title }}&#8221;</span>
                <# if ( data.artists && data.va && data.meta.artist ) { #>
                <span class="sap-playlist-item-artist"> &mdash; {{ data.meta.artist }}</span>
                <# } #>
            <# } #>
        </a>

        <# if ( data.meta.length_formatted ) { #>
        <div class="sap-playlist-item-length">
            {{ data.meta.length_formatted }}
        </div>
        <# } #>

        <# if ( data.download ) { #>
        <div class="sap-playlist-item-download">
            <a title="<?php _e("Download This Track", "smart-audio-playlists"); ?>" href="{{ data.src }}"><span class="dashicons dashicons-download"></span></a>
        </div>
        <# } #>

        <# if ( data.played ) { #>
        <div class="sap-playlist-item-played">
            <a title="<?php _e("Show Track Statistics", "smart-audio-playlists"); ?>" href="#"><span class="dashicons dashicons-chart-bar"></span></a>
        </div>

        <div class="sap-playlist-item-statistics">
            <?php _e("Track Played Total", "smart-audio-playlists"); ?>: <span>{{ data.played.total }}</span> | 
            <?php _e("Today", "smart-audio-playlists"); ?>: <span>{{ data.played.today }}</span> | 
            <?php _e("This Week", "smart-audio-playlists"); ?>: <span>{{ data.played.week }}</span> | 
            <?php _e("This Month", "smart-audio-playlists"); ?>: <span>{{ data.played.month }}</span>
        </div>
        <# } #>
    </div>
</script>
<?php }

function sap_render_player($playlist, $args = array(), $echo = true) {
    static $instance = 0;
    $instance++;

    if (is_numeric($playlist)) {
        $playlist = sap_playlist($playlist);
    }

    $initial_volume = number_format(smart_sap_core()->get('player_initial_volume') / 100, 1);

    $defaults = array(
        'source' => 'page',
        'tracklist' => true,
        'artists' => true,
        'autoplay' => false,
        'playlist_class' => '',
        'max_width' => '',
        'link_playlist' => false,
        'volume' => $initial_volume,
        'thumbnail' => 'sap-track-image',
        'skin' => smart_sap_core()->get('player_skin'),
        'style' => smart_sap_core()->get('player_style'),
        'trackstats' => $playlist->show_track_statistics,
        'tracknumbers' => $playlist->show_track_numbers,
        'images' => $playlist->show_track_images,
        'download' => $playlist->show_track_download,
        'metadata' => $playlist->show_metadata,
        'native_ipad' => smart_sap_core()->get('player_native_ipad'),
        'native_iphone' => smart_sap_core()->get('player_native_iphone'),
        'native_android' => smart_sap_core()->get('player_native_android'),
        'track_use_playlist_image' => smart_sap_core()->get('track_use_playlist_image'),
        'track_use_default_image' => smart_sap_core()->get('track_use_default_image'),
        'tracklist_toggle' => smart_sap_core()->get('player_toggle'),
        'tracklist_toggle_nohide' => smart_sap_core()->get('player_toggle_nohide'),
        'tracklist_toggle_default' => smart_sap_core()->get('player_toggle_hidden') ? 'hide' : 'show',
        'link_playlist_text' => smart_sap_core()->get('player_link_playlist_text'),
        'tracklist_toggle_open' => smart_sap_core()->get('player_tracklist_toggle_open'),
        'tracklist_toggle_close' => smart_sap_core()->get('player_tracklist_toggle_close'),
        'features' => smart_sap_core()->player_controls()
    );

    $args = wp_parse_args($args, $defaults);

    $args['variousartists'] = $playlist->various_artists == 1;
    $args['compilation'] = $playlist->compilation == 1;

    extract($args);

    $json = array('type' => 'audio', 
        'skin' => 'mejs-'.$skin, 
        'metadata' => $metadata,
        'playlist' => $playlist->id,
        'source' => $source,
        'volume' => $volume,
        'log' => 0,
        'features' => smart_sap_core()->arrange_player_controls($features)
    );

    foreach (array('autoplay', 'tracklist', 'tracknumbers', 'images', 'artists', 'download', 'variousartists', 'compilation', 'native_ipad', 'native_iphone', 'native_android') as $key) {
        $json[$key] = filter_var($$key, FILTER_VALIDATE_BOOLEAN);
    }

    $tracks = array();
    $noscript = array();

    $input = $playlist->get_tracks();

    foreach ($input as $track) {
        $url = ''; $link = '';

        if ($track->type == 'remote') {
            $url = $track->url;
            $link = '<a href="'.$url.'">'.$url.'</a>';
        } else if ($track->type == 'local') {
            $url = wp_get_attachment_url($track->post);
            $link = wp_get_attachment_link($track->post);
        }

        if ($url == '' || $url === false) {
            continue;
        }

        $noscript[] = '<li>'.$link.'</li>';

        $track_title = trim($track->title);

        if ($track_title == '') {
            $track_title = pathinfo($url, PATHINFO_FILENAME);
        }

        $T = array(
            'src' => $url,
            'track' => $track->id,
            'type' => 'audio',
            'title' => $track_title,
            'download' => $download,
            'tracks' => $tracknumbers,
            'va' => $variousartists,
            'plays' => 0,
            'caption' => '',
            'meta' => array(
                'album' => $track->album,
                'artist' => $track->artist,
                'year' => $track->year,
                'length_formatted' => $track->length
            )
        );

        if ($trackstats) {
            $stats = $track->stats();

            $T['played'] = array(
                'total' => $stats['played_total'].' '._n("time", "times", $stats['played_total'], "smart-audio-playlists"),
                'month' => $stats['played_month'].' '._n("time", "times", $stats['played_month'], "smart-audio-playlists"),
                'week' => $stats['played_week'].' '._n("time", "times", $stats['played_week'], "smart-audio-playlists"),
                'today' => $stats['played_day'].' '._n("time", "times", $stats['played_day'], "smart-audio-playlists")
            );
        }

        if ($images) {
            $thumb = $track->thumb_id();

            if ($thumb > 0) {
                list($src, $width, $height) = wp_get_attachment_image_src($thumb, 'full');
                $T['image'] = compact('src', 'width', 'height');

                list($src, $width, $height) = wp_get_attachment_image_src($thumb, $thumbnail);
                $T['thumb'] = compact('src', 'width', 'height');
            } else {
                if ($track_use_playlist_image) {
                    list($src, $width, $height) = wp_get_attachment_image_src($playlist->id, 'full');
                    $T['image'] = compact('src', 'width', 'height');

                    list($src, $width, $height) = wp_get_attachment_image_src($playlist->id, $thumbnail);
                    $T['thumb'] = compact('src', 'width', 'height');
                }

                if (!isset($T['image']) || $T['image']['src'] == '' && $track_use_default_image) {
                    $src = wp_mime_type_icon('audio/mpeg');
                    $width = 48; $height = 64;

                    $T['image'] = compact('src', 'width', 'height');
                    $T['thumb'] = compact('src', 'width', 'height');
                }
            }

            if (isset($T['image']) && $T['image']['src'] == '') {
                unset($T['image']);
                unset($T['thumb']);
            }
        }

        $tracks[] = $T;
    }

    $json['tracks'] = apply_filters('sap-playlist-player-tracks', $tracks, $playlist);
    $json = apply_filters('sap-playlist-player-object', $json, $playlist);

    $classes = array('sap-playlist-player', 'sap-playlist-'.$playlist->id, 'sap-instance-'.$instance, $style, $playlist_class);
    $classes = apply_filters('sap-playlist-player-css', $classes, $playlist);
    $classes = array_filter($classes);

    $styles = array();
    if ($max_width != '') {
        $styles[] = 'max-width: '.$max_width.';';
        $styles[] = 'width: 100%;';
    }

    $track_styles = $toggle_open = $toggle_close = '';
    if ($tracklist_toggle) {
        if ($tracklist_toggle_default == 'hide') {
            $track_styles = ' style="display: none;"';
            $toggle_close = ' style="display: none;"';
        } else {
            $toggle_open = ' style="display: none;"';
        }
    }

    $render = '<div class="'.join(' ', $classes).'" style="'.join('', $styles).'">';
        $render.= '<script type="application/json">'.json_encode($json).'</script>';
        $render.= '<noscript><ol>'.join('', $noscript).'</ol></noscript>';
        $render.= '<div class="sap-playlist-header">';
            if ($metadata) {
                $render.= '<div class="sap-playlist-current-item"></div>';
            }
        $render.= '</div>';

        if ($tracklist_toggle) {
            $render.= '<div class="sap-playlist-ctrl sap-ctrl-open"'.$toggle_open.'><a class="sap-tracklist-open" href="#">'.$tracklist_toggle_open.'</a></div>';
        }

        $render.= '<div class="sap-playlist-tracks"'.$track_styles.'></div>';

        if ($tracklist_toggle && !$tracklist_toggle_nohide) {
            $render.= '<div class="sap-playlist-ctrl sap-ctrl-close"'.$toggle_close.'><a class="sap-tracklist-close" href="#">'.$tracklist_toggle_close.'</a></div>';
        }

        if ($link_playlist) {
            $render.= '<div class="sap-playlist-ctrl sap-ctrl-link"><a href="'.get_permalink($playlist->id).'">'.$link_playlist_text.'</a></div>';
        }

    $render.= '</div>';

    do_action('sap_playlist_scripts', $skin);

    if ($echo) {
        echo $render;
    } else {
        return $render;
    }
}

?>