<?php

if (!defined('ABSPATH')) exit;

class sap_playlist_core {
    public $admin = false;

    public $id;
    public $statistics_keys = array(
        'played_total', 
        'played_day', 
        'played_week', 
        'played_month', 
        'played_year', 
        'last_played'
    );
    public $meta = array(
        'tracks' => array(),
        'statistics' => array(),
        'total_time' => 0,
        'heading' => '',
        'type' => 'n-a',
        'edition' => 'n-a',
        'various_artists' => 0,
        'compilation' => 0,
        'release_date_year' => 2014,
        'release_date_month' => 0,
        'release_date_day' => 0,
        'show_about' => -1,
        'show_statistics' => -1,
        'show_metadata' => -1,
        'show_track_statistics' => -1,
        'show_track_images' => -1,
        'show_track_numbers' => -1,
        'show_track_download' => -1
    );

    function __construct($id = 0) {
        $this->id = intval($id);

        $this->meta = array_merge($this->meta, smart_sap_core()->meta_fields('playlist'));

        if ($this->id > 0) {
            global $wpdb;

            $sql = "select meta_key, meta_value from ".$wpdb->postmeta." where post_id = ".$this->id." and meta_key like '_sap_playlist_%'";
            $raw = $wpdb->get_results($sql);

            $statistics = array();

            foreach ($raw as $r) {
                $key = substr($r->meta_key, 14);
                $val = maybe_unserialize($r->meta_value);

                if (in_array($key, $this->statistics_keys)) {
                    $statistics[$key] = $val;
                } else {
                    $this->meta[$key] = $val;
                }
            }

            $this->meta['statistics'] = array_merge($this->meta['statistics'], $statistics);
        }
    }

    public static function instance($id) {
        static $sap_playlists = array();

        if (!isset($sap_playlists[$id])) {
            $sap_playlists[$id] = new sap_playlist_core($id);
        }

        return $sap_playlists[$id];
    }

    public function __get($name) {
        if (isset($this->meta[$name])) {
            if (substr($name, 0, 4) == 'show' && !$this->admin) {
                return $this->show_override($name);
            } else {
                return $this->meta[$name];
            }
        } else {
            return '';
        }
    }

    public function __set($name, $value) {
        $this->meta[$name] = $value;
    }

    public function show_override($name) {
        if (!isset($this->meta[$name]) || $this->meta[$name] == -1) {
            $key = substr($name, 5);

            return smart_sap_core()->get('player_visible_'.$key);
        } else {
            return $this->meta[$name] == 1;
        }
    }

    public function stats() {
        $defaults = array(
            '_day' => -1,
            '_week' => -1,
            '_month' => -1,
            '_year' => -1,
            'played_total' => 0,
            'played_day' => 0,
            'played_week' => 0,
            'played_month' => 0,
            'played_year' => 0,
            'last_played' => 0
        );

        return wp_parse_args($this->meta['statistics'], $defaults);
    }

    public function update($data) {
        $processor = smart_sap_core()->meta_processor(array('string' => array(),
            'int' => array('total_time', 'release_date_year', 'release_date_month', 'release_date_day'),
            'bool' => array('various_artists', 'compilation'),
            'boolint' => array('show_track_download', 'show_track_images', 'show_track_numbers', 'show_track_statistics', 'show_about', 'show_statistics', 'show_metadata'),
            'array' => array('tracks', 'statistics'),
            'html' =>  array('heading')
        ), 'playlist');

        foreach ($this->meta as $key => $value) {
            if (in_array($key, $processor['int'])) {
                $value = isset($data[$key]) ? intval($data[$key]) : intval($value);
            } else if (in_array($key, $processor['bool'])) {
                $value = isset($data[$key]) ? 1 : 0;
            } else if (in_array($key, $processor['boolint'])) {
                $value = isset($data[$key]) ? intval($data[$key]) : -1;
            } else if (in_array($key, $processor['array'])) {
                $value = isset($data[$key]) ? (array)$data[$key] : $value;
            } else if (in_array($key, $processor['html'])) {
                $value = isset($data[$key]) ? stripslashes($data[$key]) : $value;
            } else if (in_array($key, $processor['string'])) {
                $value = isset($data[$key]) ? strip_tags(stripslashes($data[$key])) : $value;
            }

            $this->meta[$key] = $value;
        }

        $this->save();
    }

    public function save() {
        foreach ($this->meta as $key => $value) {
            update_post_meta($this->id, '_sap_playlist_'.$key, $value);
        }
    }

    public function running_length() {
        $result = count($this->tracks).' ';
        $result.= _n(smart_sap_core()->get('cpt_track_singular'), smart_sap_core()->get('cpt_track_plural'), count($this->tracks), "smart-audio-playlists");

        return $result;
    }

    public function running_time() {
        $min = floor($this->total_time / 60);
        $sec = $this->total_time - $min * 60;

        $min = $min < 10 ? '0'.$min : $min;
        $sec = $sec < 10 ? '0'.$sec : $sec;

        return $min.':'.$sec;
    }

    public function release_date($format = '') {
        if ($this->release_date_month == 0 && $this->release_date_day == 0) {
            return $this->release_date_year;
        } else {
            if ($this->release_date_day == 0) {
                $format = apply_filters('sap_playlist_release_date_format_month', 'F Y');

                $timestamp = mktime(0, 0, 0, $this->release_date_month, 1, $this->release_date_year);

                return date($format, $timestamp);
            } else {
                $format = $format == '' ? get_option('date_format') : $format;

                $format = apply_filters('sap_playlist_release_date_format_full', $format);

                $timestamp = mktime(0, 0, 0, $this->release_date_month, $this->release_date_day, $this->release_date_year);

                return date($format, $timestamp);
            }
        }
    }

    public function heading_content() {
        $content = apply_filters('the_content', $this->heading);
	return str_replace(']]>', ']]&gt;', $content);
    }

    public function reset() {
        delete_post_meta($this->id, '_sap_playlist_statistics');
    }

    public function to_archive($period, $value, $count) {
        $key = '_sap_archive_'.$period.'_'.$value;

        update_post_meta($this->id, $key, $count);
    }

    public function played() {
        $stats = $this->stats();

        $day = date('j');
        $week = date('W');
        $month = date('m');
        $year = date('Y');

        if ($stats['_day'] != $day) {
            $stats['_day'] = $day;
            $stats['played_day'] = 0;
        }

        if ($stats['_week'] != $week) {
            if ($stats['_week'] != -1) {
                $this->to_archive('week', $stats['_year'].'-'.$stats['_week'], $stats['played_week']);
            }

            $stats['_week'] = $week;
            $stats['played_week'] = 0;
        }

        if ($stats['_month'] != $month) {
            if ($stats['_month'] != -1) {
                $this->to_archive('month', $stats['_year'].'-'.$stats['_month'], $stats['played_month']);
            }

            $stats['_month'] = $month;
            $stats['played_month'] = 0;
        }

        if ($stats['_year'] != $year) {
            if ($stats['_year'] != -1) {
                $this->to_archive('year', $stats['_year'], $stats['played_year']);
            }

            $stats['_year'] = $year;
            $stats['played_year'] = 0;
        }

        $stats['played_day']++;
        $stats['played_week']++;
        $stats['played_month']++;
        $stats['played_year']++;
        $stats['played_total']++;
        $stats['last_played'] = time();

        do_action('sap_played_playlist', $this->id, $stats);

        foreach ($this->statistics_keys as $key) {
            update_post_meta($this->id, '_sap_playlist_'.$key, $stats[$key]);
            unset($stats[$key]);
        }

        update_post_meta($this->id, '_sap_playlist_statistics', $stats);

        $this->meta['statistics'] = $stats;
    }

    public function get_tracks() {
        $tracks = array();

        foreach ($this->tracks as $id) {
            $tracks[] = sap_track($id);
        }

        return $tracks;
    }

    public function log_add($source) {
        global $wpdb;

        $entry = array(
            'source' => $source,
            'post_id' => $this->id,
            'user_id' => get_current_user_id(),
            'played' => current_time('mysql'),
            'ip' => smart_sap_core()->ip()
        );

        $wpdb->insert($wpdb->prefix.'sap_play_log', $entry);
        return $wpdb->insert_id;
    }
}

function sap_playlist($id = 0) {
    return sap_playlist_core::instance($id);
}

?>