<?php

if (!defined('ABSPATH')) exit;

function is_smart_audio_playlists() {
    global $wp_query;

    $is = is_singular('sapplaylist') || is_post_type_archive('sapplaylist') ||
          $wp_query->is_sap_taxonomy_index || $wp_query->is_sap_taxonomy_archive;

    return $is;
}

function sap_get_terms($taxonomies, $args = '') {
    global $wpdb;
    $empty_array = array();

    $single_taxonomy = false;
    if (!is_array($taxonomies)) {
        $single_taxonomy = true;
        $taxonomies = array($taxonomies);
    }

    foreach ((array)$taxonomies as $taxonomy) {
        if (!taxonomy_exists($taxonomy)) {
            $error = new WP_Error('invalid_taxonomy', __("Invalid Taxonomy", "smart-audio-playlists"));
            return $error;
        }
    }

    $in_taxonomies = "'" . implode("', '", $taxonomies) . "'";

    $defaults = array('orderby' => 'name', 'order' => 'ASC', 'post_types' => array(),
        'hide_empty' => true, 'exclude' => array(), 'exclude_tree' => array(), 'include' => array(),
        'number' => '', 'fields' => 'all', 'slug' => '', 'parent' => '',
        'hierarchical' => true, 'child_of' => 0, 'get' => '', 'name__like' => '',
        'pad_counts' => false, 'offset' => '', 'search' => '');
    $args = wp_parse_args($args, $defaults);

    if (!is_array($args['post_types'])) {
        if (!empty($args['post_types'])) {
            $args['post_types'] = (array)$args['post_types'];
        } else {
            $args['post_types'] = array();
        }
    }

    $args['number'] = absint($args['number']);
    $args['offset'] = absint($args['offset']);

    if (!$single_taxonomy || !is_taxonomy_hierarchical($taxonomies[0]) || '' !== $args['parent']) {
        $args['child_of'] = 0;
        $args['hierarchical'] = false;
        $args['pad_counts'] = false;
    }

    if ('all' == $args['get']) {
        $args['child_of'] = 0;
        $args['hide_empty'] = 0;
        $args['hierarchical'] = false;
        $args['pad_counts'] = false;
    }

    extract($args, EXTR_SKIP);

    $in_post_types = "'" . implode("', '", $post_types) . "'";

    if ($child_of) {
        $hierarchy = _get_term_hierarchy($taxonomies[0]);
        if (!isset($hierarchy[$child_of]))
            return $empty_array;
    }

    if ($parent) {
        $hierarchy = _get_term_hierarchy($taxonomies[0]);
        if (!isset($hierarchy[$parent]))
            return $empty_array;
    }

    $filter_key = (has_filter('list_terms_exclusions')) ? serialize($GLOBALS['wp_filter']['list_terms_exclusions']) : '';
    $key = md5(serialize(compact(array_keys($args))).serialize($taxonomies).$filter_key);
    $last_changed = wp_cache_get('last_changed', 'terms');

    if (!$last_changed) {
        $last_changed = time();
        wp_cache_set('last_changed', $last_changed, 'terms');
    }

    $cache_key = "sap_get_terms:$key:$last_changed";
    $cache = wp_cache_get($cache_key, 'terms');

    if (false !== $cache) {
        $cache = apply_filters('get_terms', $cache, $taxonomies, $args);
        return $cache;
    }

    $_orderby = strtolower($orderby);
    if ('count' == $_orderby && empty($post_types)) {
        $orderby = 'tt.count';
    }
    if ('count' == $_orderby && !empty($post_types)) {
        $orderby = 'count(*)';
    } else if ('name' == $_orderby) {
        $orderby = 't.name';
    } else if ('slug' == $_orderby) {
        $orderby = 't.slug';
    } else if ('term_group' == $_orderby) {
        $orderby = 't.term_group';
    } else if ('rand' == $_orderby) {
        $orderby = 'rand()';
        $order = '';
    } else if ('none' == $_orderby) {
        $orderby = '';
    } else if (empty($_orderby) || 'id' == $_orderby) {
        $orderby = 't.term_id';
    }

    $orderby = apply_filters('get_terms_orderby', $orderby, $args);

    if (!empty($orderby)) {
        $orderby = "ORDER BY $orderby";
    } else {
        $order = '';
    }

    $where = $group_by = $inclusions = '';
    if (!empty($post_types)) {
        $group_by = ' GROUP BY t.term_id';
    }

    if (!empty($include)) {
        $exclude = $exclude_tree = '';
        $interms = wp_parse_id_list($include);

        foreach ($interms as $interm) {
            if (empty($inclusions)) {
                $inclusions = ' AND (t.term_id = '.intval($interm).' ';
            } else {
                $inclusions.= ' OR t.term_id = '.intval($interm).' ';
            }
        }
    }

    if (!empty($inclusions)) {
        $inclusions.= ')';
    }

    $where.= $inclusions;
    $exclusions = '';
    if (!empty($exclude_tree)) {
        $excluded_trunks = wp_parse_id_list($exclude_tree);

        foreach ($excluded_trunks as $extrunk) {
            $excluded_children = (array)get_terms($taxonomies[0], array('child_of' => intval($extrunk), 'fields' => "ids"));
            $excluded_children[] = $extrunk;

            foreach($excluded_children as $exterm) {
                if (empty($exclusions))
                    $exclusions = ' AND ( t.term_id <> '.intval($exterm).' ';
                else
                    $exclusions.= ' AND t.term_id <> '.intval($exterm).' ';
            }
        }
    }

    if (!empty($exclude)) {
        $exterms = wp_parse_id_list($exclude);
        foreach ($exterms as $exterm) {
            if (empty($exclusions))
                $exclusions = ' AND ( t.term_id <> '.intval($exterm).' ';
            else
                $exclusions.= ' AND t.term_id <> '.intval($exterm).' ';
        }
    }

    if (!empty($exclusions)) {
        $exclusions.= ')';
    }

    $exclusions = apply_filters("list_terms_exclusions", $exclusions, $args );
    $where.= $exclusions;

    if (!empty($slug)) {
        $slug = sanitize_title($slug);
        $where.= " AND t.slug = '$slug'";
    }

    if (!empty($name__like))
        $where.= " AND t.name LIKE '{$name__like}%'";

    if ('' !== $parent) {
        $parent = (int) $parent;
        $where.= " AND tt.parent = '$parent'";
    }

    if ($hide_empty && !$hierarchical) {
        if (empty($post_types)) $where.= ' AND tt.count > 0';
        else $group_by.= ' HAVING count(*) > 0';
    }

    if (!empty($number) && !$hierarchical && empty($child_of) && "" === $parent) {
        if ($offset)
            $limit = 'LIMIT '.$offset.', '.$number;
        else
            $limit = 'LIMIT '.$number;
    } else {
        $limit = '';
    }

    if (!empty($search)) {
        $search = like_escape($search);
        $where.= " AND (t.name LIKE '%$search%')";
    }

    if (!empty($post_types)) {
        $where.= " AND p.post_type in ($in_post_types)";
    }

    $selects = array();
    switch ($fields) {
        case 'all':
            if (empty($post_types)) {
                $selects = array('t.*', 'tt.*');
            } else {
                $selects = array('t.*', 'tt.term_taxonomy_id', 'tt.taxonomy', 'tt.description', 'tt.parent', 'count(*) as count');
            }
            break;
        case 'ids':
        case 'id=>parent':
            if (empty($post_types)) {
                $selects = array('t.term_id', 'tt.parent', 'tt.count');
            } else {
                $selects = array('t.term_id', 'tt.parent', 'count(*) as count');
            }
            break;
        case 'names':
            if (empty($post_types)) {
                $selects = array('t.term_id', 'tt.parent', 'tt.count', 't.name');
            } else {
                $selects = array('t.term_id', 'tt.parent', 'count(*) as count', 't.name');
            }
            break;
        case 'count':
           $orderby = '';
           $order = '';
           $selects = array('COUNT(*)');
    }
    $select_this = implode(', ', apply_filters('get_terms_fields', $selects, $args));

    if (empty($post_types)) {
        $query = "SELECT $select_this FROM $wpdb->terms AS t INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id WHERE tt.taxonomy IN ($in_taxonomies) $where $orderby $order $limit";
    } else {
        $query = "SELECT $select_this FROM $wpdb->terms AS t INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id INNER JOIN $wpdb->term_relationships AS tr ON tr.term_taxonomy_id = tt.term_taxonomy_id INNER JOIN $wpdb->posts AS p ON p.ID = tr.object_id WHERE tt.taxonomy IN ($in_taxonomies) $where $group_by $orderby $order $limit";
    }

    if ('count' == $fields) {
        $term_count = $wpdb->get_var($query);
        return $term_count;
    }

    $terms = $wpdb->get_results($query);

    if ('all' == $fields) {
        update_term_cache($terms);
    }

    if (empty($terms)) {
        wp_cache_add($cache_key, array(), 'terms');

        $terms = apply_filters('get_terms', array(), $taxonomies, $args);
        return $terms;
    }

    if ($child_of) {
        $children = _get_term_hierarchy($taxonomies[0]);
        if (!empty($children))
            $terms = & _get_term_children($child_of, $terms, $taxonomies[0]);
    }

    if ($pad_counts && 'all' == $fields) {
        _pad_term_counts($terms, $taxonomies[0]);
    }

    if ($hierarchical && $hide_empty && is_array($terms)) {
        foreach ($terms as $k => $term) {
            if (!$term->count) {
                $children = _get_term_children($term->term_id, $terms, $taxonomies[0]);

                if (is_array($children)) {
                    foreach ($children as $child)
                        if ($child->count) continue 2;
                }

                unset($terms[$k]);
            }
        }
    }

    reset($terms);

    $_terms = array();
    if ('id=>parent' == $fields) {
        while ($term = array_shift($terms))
            $_terms[$term->term_id] = $term->parent;
        $terms = $_terms;
    } elseif ('ids' == $fields) {
        while ($term = array_shift($terms))
            $_terms[] = $term->term_id;
        $terms = $_terms;
    } elseif ('names' == $fields) {
        while ($term = array_shift($terms))
            $_terms[] = $term->name;
        $terms = $_terms;
    }

    if (0 < $number && intval(@count($terms)) > $number) {
        $terms = array_slice($terms, $offset, $number);
    }

    wp_cache_add($cache_key, $terms, 'terms');
    $terms = apply_filters('get_terms', $terms, $taxonomies, $args);

    return $terms;
}

function sap_terms_update_with_urls($terms, $taxonomy) {
    $_base = get_post_type_archive_link('sapplaylist');

    if ($taxonomy == 'post_tag') {
        $_slug = get_taxonomy($taxonomy)->rewrite['slug'];
    } else if ($taxonomy == 'sapgenre') {
        $_slug = smart_sap_core()->get('tax_genre_slug_single');
    } else if ($taxonomy == 'sapartist') {
        $_slug = smart_sap_core()->get('tax_artist_slug_single');
    }

    foreach ($terms as $key => $term) {
        $link = $_base.$_slug.'/'.$term->slug.'/';

        $terms[$key]->link = $link;
        $terms[$key]->id = $term->term_id;
    }

    return $terms;
}

function sap_terms_cloud($args = '') {
    $defaults = array(
        'smallest' => 10, 'largest' => 30, 'unit' => 'pt', 'number' => '', 'format' => 'flat', 
        'separator' => "\n", 'orderby' => 'name', 'order' => 'ASC', 'exclude' => '', 'include' => '', 
        'taxonomy' => 'post_tag', 'post_types' => '', 'echo' => true);
    $args = wp_parse_args($args, $defaults);

    $terms = sap_get_terms($args['taxonomy'], array_merge($args, array('orderby' => 'count', 'order' => 'DESC')));

    if (empty($terms) || is_wp_error($terms)) {
        return;
    }

    $terms = sap_terms_update_with_urls($terms, $args['taxonomy']);

    $return = wp_generate_tag_cloud($terms, $args);

    $return = apply_filters('sap_terms_cloud', $return, $args);

    if ('array' == $args['format'] || empty($args['echo'])) {
        return $return;
    }

    echo $return;
}

function sap_template_part($slug, $name, $code = null) {
    $templates = array();

    if (is_null($code)) {
        if ($slug == 'single') {
            $code = smart_sap_core()->get('templates_playlist_single');
        } else if ($slug == 'archive') {
            $code = smart_sap_core()->get('templates_playlist_archive');
        } else if ($slug == 'index') {
            $tax = get_query_var('sapindex');

            $code = smart_sap_core()->get('templates_playlist_index_'.$tax);
        }
    }

    $code = (string)$code;

    if ('' !== $code) {
        $templates[] = "sap-{$slug}-{$name}-{$code}.php";
    }

    $templates[] = "sap-{$slug}-{$name}.php";

    sap_locate_template($templates, true, false);
}

function sap_locate_theme_template($template_names, $load = false, $require_once = true) {
    $theme = get_template();

    $located = apply_filters('sap_locate_theme_template', '', $template_names, $theme);

    if ($located == '') {
        foreach ((array)$template_names as $template_name) {
            if (!$template_name) {
                continue;
            }

            if (file_exists(STYLESHEETPATH.'/'.$template_name)) {
                $located = STYLESHEETPATH.'/'.$template_name;
                break;
            } else if (file_exists(TEMPLATEPATH.'/'.$template_name)) {
                $located = TEMPLATEPATH.'/'.$template_name;
                break;
            } else if (file_exists(SAP_PATH.'themes/'.$theme.'/'.$template_name)) {
                $located = SAP_PATH.'themes/'.$theme.'/'.$template_name;
                break;
            }
        }
    }

    if ($located == '') {
        foreach ((array)$template_names as $template_name) {
            if (!$template_name) {
                continue;
            }

            if (file_exists(SAP_PATH.'themes/'.$template_name)) {
                $located = SAP_PATH.'themes/'.$template_name;
                break;
            }
        }
    }
    
    if ($load && '' != $located) {
        load_template($located, $require_once);
    }

    return $located;
}

function sap_locate_template($template_names, $load = false, $require_once = true) {
    $located = apply_filters('sap_locate_template', '', $template_names);

    if ($located == '') {
        foreach ((array)$template_names as $template_name) {
            if (!$template_name) {
                continue;
            }

            if (file_exists(STYLESHEETPATH.'/'.$template_name)) {
                $located = STYLESHEETPATH.'/'.$template_name;
                break;
            } else if (file_exists(TEMPLATEPATH.'/'.$template_name)) {
                $located = TEMPLATEPATH.'/'.$template_name;
                break;
            } else if (file_exists(SAP_PATH.'templates/'.$template_name)) {
                $located = SAP_PATH.'templates/'.$template_name;
                break;
            }
        }
    }

    if ($load && '' != $located) {
        load_template($located, $require_once);
    }

    return $located;
}

function sap_terms_list($id, $taxonomy) {
    $_base = get_post_type_archive_link('sapplaylist');

    if ($taxonomy == 'post_tag') {
        $_slug = get_taxonomy($taxonomy)->rewrite['slug'];
    } else if ($taxonomy == 'sapgenre') {
        $_slug = smart_sap_core()->get('tax_genre_slug_single');
    } else if ($taxonomy == 'sapartist') {
        $_slug = smart_sap_core()->get('tax_artist_slug_single');
    }

    $terms = get_the_terms($id, $taxonomy);

    if (is_wp_error($terms) || empty($terms)) {
        return array();
    }

    $term_links = array();

    foreach ($terms as $term) {
        $link = $_base.$_slug.'/'.$term->slug.'/';
        $term_links[] = '<a href="'.esc_url($link).'" rel="tag">'.$term->name.'</a>';
    }

    return $term_links;
}

?>